@@ -91,7 +91,6 @@ See :hg:`help patterns` for more information about the glob patterns
 used.
 """
 
-from __future__ import absolute_import
 
 import os
 import re
@@ -186,7 +185,7 @@ filters = {
 }
 
 
-class eolfile(object):
+class eolfile:
     def __init__(self, ui, root, data):
         self._decode = {
             b'LF': b'to-lf',
@@ -310,7 +309,7 @@ def _checkhook(ui, repo, node, headsonly):
     ensureenabled(ui)
     files = set()
     revs = set()
-    for rev in pycompat.xrange(repo[node].rev(), len(repo)):
+    for rev in range(repo[node].rev(), len(repo)):
         revs.add(rev)
         if headsonly:
             ctx = repo[rev]
@@ -379,7 +378,7 @@ def reposetup(ui, repo):
 
     if not repo.local():
         return
-    for name, fn in pycompat.iteritems(filters):
+    for name, fn in filters.items():
         repo.adddatafilter(name, fn)
 
     ui.setconfig(b'patch', b'eol', b'auto', b'eol')
@@ -423,7 +422,7 @@ def reposetup(ui, repo):
                 try:
                     wlock = self.wlock()
                     for f in self.dirstate:
-                        if self.dirstate[f] != b'n':
+                        if not self.dirstate.get_entry(f).maybe_clean:
                             continue
                         if oldeol is not None:
                             if not oldeol.match(f) and not neweol.match(f):
@@ -442,7 +441,7 @@ def reposetup(ui, repo):
                                 continue
                         # all normal files need to be looked at again since
                         # the new .hgeol file specify a different filter
-                        self.dirstate.normallookup(f)
+                        self.dirstate.set_possibly_dirty(f)
                     # Write the cache to update mtime and cache .hgeol
                     with self.vfs(b"eol.cache", b"w") as f:
                         f.write(hgeoldata)
