@@ -154,7 +154,6 @@ web.baseurl
   references. See also ``notify.strip``.
 
 '''
-from __future__ import absolute_import
 
 import email.errors as emailerrors
 import email.utils as emailutils
@@ -190,64 +189,104 @@ configtable = {}
 configitem = registrar.configitem(configtable)
 
 configitem(
-    b'notify', b'changegroup', default=None,
+    b'notify',
+    b'changegroup',
+    default=None,
 )
 configitem(
-    b'notify', b'config', default=None,
+    b'notify',
+    b'config',
+    default=None,
 )
 configitem(
-    b'notify', b'diffstat', default=True,
+    b'notify',
+    b'diffstat',
+    default=True,
 )
 configitem(
-    b'notify', b'domain', default=None,
+    b'notify',
+    b'domain',
+    default=None,
 )
 configitem(
-    b'notify', b'messageidseed', default=None,
+    b'notify',
+    b'messageidseed',
+    default=None,
 )
 configitem(
-    b'notify', b'fromauthor', default=None,
+    b'notify',
+    b'fromauthor',
+    default=None,
 )
 configitem(
-    b'notify', b'incoming', default=None,
+    b'notify',
+    b'incoming',
+    default=None,
 )
 configitem(
-    b'notify', b'maxdiff', default=300,
+    b'notify',
+    b'maxdiff',
+    default=300,
 )
 configitem(
-    b'notify', b'maxdiffstat', default=-1,
+    b'notify',
+    b'maxdiffstat',
+    default=-1,
 )
 configitem(
-    b'notify', b'maxsubject', default=67,
+    b'notify',
+    b'maxsubject',
+    default=67,
 )
 configitem(
-    b'notify', b'mbox', default=None,
+    b'notify',
+    b'mbox',
+    default=None,
 )
 configitem(
-    b'notify', b'merge', default=True,
+    b'notify',
+    b'merge',
+    default=True,
 )
 configitem(
-    b'notify', b'outgoing', default=None,
+    b'notify',
+    b'outgoing',
+    default=None,
 )
 configitem(
-    b'notify', b'reply-to-predecessor', default=False,
+    b'notify',
+    b'reply-to-predecessor',
+    default=False,
 )
 configitem(
-    b'notify', b'sources', default=b'serve',
+    b'notify',
+    b'sources',
+    default=b'serve',
 )
 configitem(
-    b'notify', b'showfunc', default=None,
+    b'notify',
+    b'showfunc',
+    default=None,
 )
 configitem(
-    b'notify', b'strip', default=0,
+    b'notify',
+    b'strip',
+    default=0,
 )
 configitem(
-    b'notify', b'style', default=None,
+    b'notify',
+    b'style',
+    default=None,
 )
 configitem(
-    b'notify', b'template', default=None,
+    b'notify',
+    b'template',
+    default=None,
 )
 configitem(
-    b'notify', b'test', default=True,
+    b'notify',
+    b'test',
+    default=True,
 )
 
 # template for single changeset can include email headers.
@@ -275,7 +314,7 @@ deftemplates = {
 }
 
 
-class notifier(object):
+class notifier:
     '''email notification class.'''
 
     def __init__(self, ui, repo, hooktype):
@@ -395,7 +434,10 @@ class notifier(object):
             if spec is None:
                 subs.add(sub)
                 continue
-            revs = self.repo.revs(b'%r and %d:', spec, ctx.rev())
+            try:
+                revs = self.repo.revs(b'%r and %d:', spec, ctx.rev())
+            except error.RepoLookupError:
+                continue
             if len(revs):
                 subs.add(sub)
                 continue
@@ -423,7 +465,7 @@ class notifier(object):
             # create fresh mime message from scratch
             # (multipart templates must take care of this themselves)
             headers = msg.items()
-            payload = msg.get_payload(decode=pycompat.ispy3)
+            payload = msg.get_payload(decode=True)
             # for notification prefer readability over data precision
             msg = mail.mimeencode(self.ui, payload, self.charsets, self.test)
             # reinstate custom headers
@@ -482,7 +524,7 @@ class notifier(object):
                 )
         msg['To'] = ', '.join(sorted(subs))
 
-        msgtext = msg.as_bytes() if pycompat.ispy3 else msg.as_string()
+        msgtext = msg.as_bytes()
         if self.test:
             self.ui.write(msgtext)
             if not msgtext.endswith(b'\n'):
@@ -539,10 +581,10 @@ class notifier(object):
 
 
 def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
-    '''send email notifications to interested subscribers.
+    """send email notifications to interested subscribers.
 
     if used as changegroup hook, send one email for all changesets in
-    changegroup. else send one email per changeset.'''
+    changegroup. else send one email per changeset."""
 
     n = notifier(ui, repo, hooktype)
     ctx = repo.unfiltered()[node]
