@@ -14,9 +14,7 @@ For more information:
 https://mercurial-scm.org/wiki/RebaseExtension
 '''
 
-from __future__ import absolute_import
 
-import errno
 import os
 
 from mercurial.i18n import _
@@ -35,6 +33,7 @@ from mercurial import (
     dirstateguard,
     error,
     extensions,
+    logcmdutil,
     merge as mergemod,
     mergestate as mergestatemod,
     mergeutil,
@@ -159,7 +158,7 @@ def _ctxdesc(ctx):
     )
 
 
-class rebaseruntime(object):
+class rebaseruntime:
     """This class is a container for rebase runtime state"""
 
     def __init__(self, repo, ui, inmemory=False, dryrun=False, opts=None):
@@ -190,18 +189,18 @@ class rebaseruntime(object):
         self.destmap = {}
         self.skipped = set()
 
-        self.collapsef = opts.get(b'collapse', False)
-        self.collapsemsg = cmdutil.logmessage(ui, opts)
-        self.date = opts.get(b'date', None)
+        self.collapsef = opts.get('collapse', False)
+        self.collapsemsg = cmdutil.logmessage(ui, pycompat.byteskwargs(opts))
+        self.date = opts.get('date', None)
 
-        e = opts.get(b'extrafn')  # internal, used by e.g. hgsubversion
+        e = opts.get('extrafn')  # internal, used by e.g. hgsubversion
         self.extrafns = [_savegraft]
         if e:
             self.extrafns = [e]
 
         self.backupf = ui.configbool(b'rewrite', b'backup-bundle')
-        self.keepf = opts.get(b'keep', False)
-        self.keepbranchesf = opts.get(b'keepbranches', False)
+        self.keepf = opts.get('keep', False)
+        self.keepbranchesf = opts.get('keepbranches', False)
         self.skipemptysuccessorf = rewriteutil.skip_empty_successor(
             repo.ui, b'rebase'
         )
@@ -243,7 +242,7 @@ class rebaseruntime(object):
         f.write(b'%d\n' % int(self.keepbranchesf))
         f.write(b'%s\n' % (self.activebookmark or b''))
         destmap = self.destmap
-        for d, v in pycompat.iteritems(self.state):
+        for d, v in self.state.items():
             oldrev = repo[d].hex()
             if v >= 0:
                 newrev = repo[v].hex()
@@ -362,10 +361,7 @@ class rebaseruntime(object):
         skippedset = set(self.obsolete_with_successor_in_destination)
         skippedset.update(self.obsolete_with_successor_in_rebase_set)
         _checkobsrebase(self.repo, self.ui, obsoleteset, skippedset)
-        allowdivergence = self.ui.configbool(
-            b'experimental', b'evolution.allowdivergence'
-        )
-        if allowdivergence:
+        if obsolete.isenabled(self.repo, obsolete.allowdivergenceopt):
             self.obsolete_with_successor_in_rebase_set = set()
         else:
             for rev in self.repo.revs(
@@ -446,8 +442,15 @@ class rebaseruntime(object):
             rebaseset = set(destmap.keys())
             rebaseset -= set(self.obsolete_with_successor_in_destination)
             rebaseset -= self.obsolete_with_successor_in_rebase_set
+            # We have our own divergence-checking in the rebase extension
+            overrides = {}
+            if obsolete.isenabled(self.repo, obsolete.createmarkersopt):
+                overrides = {
+                    (b'experimental', b'evolution.allowdivergence'): b'true'
+                }
             try:
-                rewriteutil.precheck(self.repo, rebaseset, action=b'rebase')
+                with self.ui.configoverride(overrides):
+                    rewriteutil.precheck(self.repo, rebaseset, action=b'rebase')
             except error.Abort as e:
                 if e.hint is None:
                     e.hint = _(b'use --keep to keep original changesets')
@@ -501,7 +504,7 @@ class rebaseruntime(object):
             # commits.
             self.storestatus(tr)
 
-        cands = [k for k, v in pycompat.iteritems(self.state) if v == revtodo]
+        cands = [k for k, v in self.state.items() if v == revtodo]
         p = repo.ui.makeprogress(
             _(b"rebasing"), unit=_(b'changesets'), total=len(cands)
         )
@@ -623,7 +626,7 @@ class rebaseruntime(object):
                 repo.ui.debug(b'resuming interrupted rebase\n')
                 self.resume = False
             else:
-                overrides = {(b'ui', b'forcemerge'): opts.get(b'tool', b'')}
+                overrides = {(b'ui', b'forcemerge'): opts.get('tool', b'')}
                 with ui.configoverride(overrides, b'rebase'):
                     try:
                         rebasenode(
@@ -670,9 +673,7 @@ class rebaseruntime(object):
             if not self.collapsef:
                 merging = p2 != nullrev
                 editform = cmdutil.mergeeditform(merging, b'rebase')
-                editor = cmdutil.getcommiteditor(
-                    editform=editform, **pycompat.strkwargs(opts)
-                )
+                editor = cmdutil.getcommiteditor(editform=editform, **opts)
                 # We need to set parents again here just in case we're continuing
                 # a rebase started with an old hg version (before 9c9cfecd4600),
                 # because those old versions would have left us with two dirstate
@@ -720,7 +721,7 @@ class rebaseruntime(object):
 
     def _finishrebase(self):
         repo, ui, opts = self.repo, self.ui, self.opts
-        fm = ui.formatter(b'rebase', opts)
+        fm = ui.formatter(b'rebase', pycompat.byteskwargs(opts))
         fm.startitem()
         if self.collapsef:
             p1, p2, _base = defineparents(
@@ -731,7 +732,7 @@ class rebaseruntime(object):
                 self.skipped,
                 self.obsolete_with_successor_in_destination,
             )
-            editopt = opts.get(b'edit')
+            editopt = opts.get('edit')
             editform = b'rebase.collapse'
             if self.collapsemsg:
                 commitmsg = self.collapsemsg
@@ -755,7 +756,7 @@ class rebaseruntime(object):
                     self.state[oldrev] = newrev
 
         if b'qtip' in repo.tags():
-            updatemq(repo, self.state, self.skipped, **pycompat.strkwargs(opts))
+            updatemq(repo, self.state, self.skipped, **opts)
 
         # restore original working directory
         # (we do this before stripping)
@@ -1056,18 +1057,17 @@ def rebase(ui, repo, **opts):
     unresolved conflicts.
 
     """
-    opts = pycompat.byteskwargs(opts)
     inmemory = ui.configbool(b'rebase', b'experimental.inmemory')
-    action = cmdutil.check_at_most_one_arg(opts, b'abort', b'stop', b'continue')
+    action = cmdutil.check_at_most_one_arg(opts, 'abort', 'stop', 'continue')
     if action:
         cmdutil.check_incompatible_arguments(
-            opts, action, [b'confirm', b'dry_run']
+            opts, action, ['confirm', 'dry_run']
         )
         cmdutil.check_incompatible_arguments(
-            opts, action, [b'rev', b'source', b'base', b'dest']
+            opts, action, ['rev', 'source', 'base', 'dest']
         )
-    cmdutil.check_at_most_one_arg(opts, b'confirm', b'dry_run')
-    cmdutil.check_at_most_one_arg(opts, b'rev', b'source', b'base')
+    cmdutil.check_at_most_one_arg(opts, 'confirm', 'dry_run')
+    cmdutil.check_at_most_one_arg(opts, 'rev', 'source', 'base')
 
     if action or repo.currenttransaction() is not None:
         # in-memory rebase is not compatible with resuming rebases.
@@ -1075,19 +1075,19 @@ def rebase(ui, repo, **opts):
         # fail the entire transaction.)
         inmemory = False
 
-    if opts.get(b'auto_orphans'):
-        disallowed_opts = set(opts) - {b'auto_orphans'}
+    if opts.get('auto_orphans'):
+        disallowed_opts = set(opts) - {'auto_orphans'}
         cmdutil.check_incompatible_arguments(
-            opts, b'auto_orphans', disallowed_opts
+            opts, 'auto_orphans', disallowed_opts
         )
 
-        userrevs = list(repo.revs(opts.get(b'auto_orphans')))
-        opts[b'rev'] = [revsetlang.formatspec(b'%ld and orphan()', userrevs)]
-        opts[b'dest'] = b'_destautoorphanrebase(SRC)'
+        userrevs = list(repo.revs(opts.get('auto_orphans')))
+        opts['rev'] = [revsetlang.formatspec(b'%ld and orphan()', userrevs)]
+        opts['dest'] = b'_destautoorphanrebase(SRC)'
 
-    if opts.get(b'dry_run') or opts.get(b'confirm'):
+    if opts.get('dry_run') or opts.get('confirm'):
         return _dryrunrebase(ui, repo, action, opts)
-    elif action == b'stop':
+    elif action == 'stop':
         rbsrt = rebaseruntime(repo, ui)
         with repo.wlock(), repo.lock():
             rbsrt.restorestatus()
@@ -1136,7 +1136,7 @@ def rebase(ui, repo, **opts):
 
 def _dryrunrebase(ui, repo, action, opts):
     rbsrt = rebaseruntime(repo, ui, inmemory=True, dryrun=True, opts=opts)
-    confirm = opts.get(b'confirm')
+    confirm = opts.get('confirm')
     if confirm:
         ui.status(_(b'starting in-memory rebase\n'))
     else:
@@ -1193,7 +1193,7 @@ def _dryrunrebase(ui, repo, action, opts):
                     isabort=True,
                     backup=False,
                     suppwarns=True,
-                    dryrun=opts.get(b'dry_run'),
+                    dryrun=opts.get('dry_run'),
                 )
 
 
@@ -1203,9 +1203,9 @@ def _dorebase(ui, repo, action, opts, inmemory=False):
 
 
 def _origrebase(ui, repo, action, opts, rbsrt):
-    assert action != b'stop'
+    assert action != 'stop'
     with repo.wlock(), repo.lock():
-        if opts.get(b'interactive'):
+        if opts.get('interactive'):
             try:
                 if extensions.find(b'histedit'):
                     enablehistedit = b''
@@ -1231,29 +1231,27 @@ def _origrebase(ui, repo, action, opts, rbsrt):
                 raise error.InputError(
                     _(b'cannot use collapse with continue or abort')
                 )
-            if action == b'abort' and opts.get(b'tool', False):
+            if action == 'abort' and opts.get('tool', False):
                 ui.warn(_(b'tool option will be ignored\n'))
-            if action == b'continue':
+            if action == 'continue':
                 ms = mergestatemod.mergestate.read(repo)
                 mergeutil.checkunresolved(ms)
 
-            retcode = rbsrt._prepareabortorcontinue(
-                isabort=(action == b'abort')
-            )
+            retcode = rbsrt._prepareabortorcontinue(isabort=(action == 'abort'))
             if retcode is not None:
                 return retcode
         else:
             # search default destination in this space
             # used in the 'hg pull --rebase' case, see issue 5214.
-            destspace = opts.get(b'_destspace')
+            destspace = opts.get('_destspace')
             destmap = _definedestmap(
                 ui,
                 repo,
                 rbsrt.inmemory,
-                opts.get(b'dest', None),
-                opts.get(b'source', []),
-                opts.get(b'base', []),
-                opts.get(b'rev', []),
+                opts.get('dest', None),
+                opts.get('source', []),
+                opts.get('base', []),
+                opts.get('rev', []),
                 destspace=destspace,
             )
             retcode = rbsrt._preparenewrebase(destmap)
@@ -1303,19 +1301,19 @@ def _definedestmap(ui, repo, inmemory, destf, srcf, basef, revf, destspace):
     dest = None
 
     if revf:
-        rebaseset = scmutil.revrange(repo, revf)
+        rebaseset = logcmdutil.revrange(repo, revf)
         if not rebaseset:
             ui.status(_(b'empty "rev" revision set - nothing to rebase\n'))
             return None
     elif srcf:
-        src = scmutil.revrange(repo, srcf)
+        src = logcmdutil.revrange(repo, srcf)
         if not src:
             ui.status(_(b'empty "source" revision set - nothing to rebase\n'))
             return None
         # `+  (%ld)` to work around `wdir()::` being empty
         rebaseset = repo.revs(b'(%ld):: + (%ld)', src, src)
     else:
-        base = scmutil.revrange(repo, basef or [b'.'])
+        base = logcmdutil.revrange(repo, basef or [b'.'])
         if not base:
             ui.status(
                 _(b'empty "base" revision set - ' b"can't compute rebase set\n")
@@ -1323,7 +1321,7 @@ def _definedestmap(ui, repo, inmemory, destf, srcf, basef, revf, destspace):
             return None
         if destf:
             # --base does not support multiple destinations
-            dest = scmutil.revsingle(repo, destf)
+            dest = logcmdutil.revsingle(repo, destf)
         else:
             dest = repo[_destrebase(repo, base, destspace=destspace)]
             destf = bytes(dest)
@@ -1337,7 +1335,7 @@ def _definedestmap(ui, repo, inmemory, destf, srcf, basef, revf, destspace):
             # emulate the old behavior, showing "nothing to rebase" (a better
             # behavior may be abort with "cannot find branching point" error)
             bpbase.clear()
-        for bp, bs in pycompat.iteritems(bpbase):  # calculate roots
+        for bp, bs in bpbase.items():  # calculate roots
             roots += list(repo.revs(b'children(%d) & ancestors(%ld)', bp, bs))
 
         rebaseset = repo.revs(b'%ld::', roots)
@@ -1544,7 +1542,7 @@ def rebasenode(repo, rev, p1, p2, base, collapse, wctx):
         force=True,
         ancestor=base,
         mergeancestor=mergeancestor,
-        labels=[b'dest', b'source'],
+        labels=[b'dest', b'source', b'parent of source'],
         wc=wctx,
     )
     wctx.setparents(p1ctx.node(), repo[p2].node())
@@ -1650,7 +1648,7 @@ def _checkobsrebase(repo, ui, rebaseobsrevs, rebaseobsskipped):
     successors in destination or no non-obsolete successor.
     """
     # Obsolete node with successors not in dest leads to divergence
-    divergenceok = ui.configbool(b'experimental', b'evolution.allowdivergence')
+    divergenceok = obsolete.isenabled(repo, obsolete.allowdivergenceopt)
     divergencebasecandidates = rebaseobsrevs - rebaseobsskipped
 
     if divergencebasecandidates and not divergenceok:
@@ -1941,9 +1939,7 @@ def restorecollapsemsg(repo, isabort):
         f = repo.vfs(b"last-message.txt")
         collapsemsg = f.readline().strip()
         f.close()
-    except IOError as err:
-        if err.errno != errno.ENOENT:
-            raise
+    except FileNotFoundError:
         if isabort:
             # Oh well, just abort like normal
             collapsemsg = b''
@@ -2104,7 +2100,7 @@ def clearrebased(
         fl = fm.formatlist
         fd = fm.formatdict
         changes = {}
-        for oldns, newn in pycompat.iteritems(replacements):
+        for oldns, newn in replacements.items():
             for oldn in oldns:
                 changes[hf(oldn)] = fl([hf(n) for n in newn], name=b'node')
         nodechanges = fd(changes, key=b"oldnode", value=b"newnodes")
@@ -2258,7 +2254,7 @@ def summaryhook(ui, repo):
         msg = _(b'rebase: (use "hg rebase --abort" to clear broken state)\n')
         ui.write(msg)
         return
-    numrebased = len([i for i in pycompat.itervalues(state) if i >= 0])
+    numrebased = len([i for i in state.values() if i >= 0])
     # i18n: column positioning for "hg summary"
     ui.write(
         _(b'rebase: %s, %s (rebase --continue)\n')
