@@ -11,10 +11,8 @@ The :hg:`releasenotes` command provided by this extension makes the
 process simpler by automating it.
 """
 
-from __future__ import absolute_import
 
 import difflib
-import errno
 import re
 
 from mercurial.i18n import _
@@ -24,10 +22,10 @@ from mercurial import (
     cmdutil,
     config,
     error,
+    logcmdutil,
     minirst,
     pycompat,
     registrar,
-    scmutil,
     util,
 )
 from mercurial.utils import (
@@ -39,9 +37,22 @@ cmdtable = {}
 command = registrar.command(cmdtable)
 
 try:
-    import fuzzywuzzy.fuzz as fuzz
+    # Silence a warning about python-Levenshtein.
+    #
+    # We don't need the the performance that much and it get anoying in tests.
+    import warnings
 
-    fuzz.token_set_ratio
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            action="ignore",
+            message=".*python-Levenshtein.*",
+            category=UserWarning,
+            module="fuzzywuzzy.fuzz",
+        )
+
+        import fuzzywuzzy.fuzz as fuzz
+
+        fuzz.token_set_ratio
 except ImportError:
     fuzz = None
 
@@ -65,7 +76,7 @@ RE_ISSUE = br'\bissue ?[0-9]{4,6}(?![0-9])\b'
 BULLET_SECTION = _(b'Other Changes')
 
 
-class parsedreleasenotes(object):
+class parsedreleasenotes:
     def __init__(self):
         self.sections = {}
 
@@ -158,14 +169,14 @@ class parsedreleasenotes(object):
                 self.addnontitleditem(section, paragraphs)
 
 
-class releasenotessections(object):
+class releasenotessections:
     def __init__(self, ui, repo=None):
         if repo:
             sections = util.sortdict(DEFAULT_SECTIONS)
             custom_sections = getcustomadmonitions(repo)
             if custom_sections:
                 sections.update(custom_sections)
-            self._sections = list(pycompat.iteritems(sections))
+            self._sections = list(sections.items())
         else:
             self._sections = list(DEFAULT_SECTIONS)
 
@@ -663,7 +674,7 @@ def releasenotes(ui, repo, file_=None, **opts):
         return _getadmonitionlist(ui, sections)
 
     rev = opts.get(b'rev')
-    revs = scmutil.revrange(repo, [rev or b'not public()'])
+    revs = logcmdutil.revrange(repo, [rev or b'not public()'])
     if opts.get(b'check'):
         return checkadmonitions(ui, repo, sections.names(), revs)
 
@@ -676,10 +687,7 @@ def releasenotes(ui, repo, file_=None, **opts):
     try:
         with open(file_, b'rb') as fh:
             notes = parsereleasenotesfile(sections, fh.read())
-    except IOError as e:
-        if e.errno != errno.ENOENT:
-            raise
-
+    except FileNotFoundError:
         notes = parsedreleasenotes()
 
     notes.merge(ui, incoming)
