@@ -6,7 +6,6 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
 
 import copy
 import difflib
@@ -14,7 +13,7 @@ import os
 import re
 
 from ..i18n import _
-from ..node import hex, nullid, short
+from ..node import hex, short
 from ..pycompat import setattr
 
 from .common import (
@@ -57,7 +56,7 @@ def archivelist(ui, nodeid, url=None):
     allowed = ui.configlist(b'web', b'allow-archive', untrusted=True)
     archives = []
 
-    for typ, spec in pycompat.iteritems(archivespecs):
+    for typ, spec in archivespecs.items():
         if typ in allowed or ui.configbool(
             b'web', b'allow' + typ, untrusted=True
         ):
@@ -100,7 +99,7 @@ def _navseq(step, firststep=None):
         step *= 10
 
 
-class revnav(object):
+class revnav:
     def __init__(self, repo):
         """Navigation generation object
 
@@ -220,7 +219,7 @@ def _ctxsgen(context, ctxs):
 def _siblings(siblings=None, hiderev=None):
     if siblings is None:
         siblings = []
-    siblings = [s for s in siblings if s.node() != nullid]
+    siblings = [s for s in siblings if s.node() != s.repo().nullid]
     if len(siblings) == 1 and siblings[0].rev() == hiderev:
         siblings = []
     return templateutil.mappinggenerator(_ctxsgen, args=(siblings,))
@@ -316,12 +315,16 @@ def _nodenamesgen(context, f, node, name):
         yield {name: t}
 
 
-def showtag(repo, t1, node=nullid):
+def showtag(repo, t1, node=None):
+    if node is None:
+        node = repo.nullid
     args = (repo.nodetags, node, b'tag')
     return templateutil.mappinggenerator(_nodenamesgen, args=args, name=t1)
 
 
-def showbookmark(repo, t1, node=nullid):
+def showbookmark(repo, t1, node=None):
+    if node is None:
+        node = repo.nullid
     args = (repo.nodebookmarks, node, b'bookmark')
     return templateutil.mappinggenerator(_nodenamesgen, args=args, name=t1)
 
@@ -717,7 +720,7 @@ def _getcompblockgen(context, leftlines, rightlines, opcodes):
         len1 = lhi - llo
         len2 = rhi - rlo
         count = min(len1, len2)
-        for i in pycompat.xrange(count):
+        for i in range(count):
             yield _compline(
                 type=type,
                 leftlineno=llo + i + 1,
@@ -726,7 +729,7 @@ def _getcompblockgen(context, leftlines, rightlines, opcodes):
                 rightline=rightlines[rlo + i],
             )
         if len1 > len2:
-            for i in pycompat.xrange(llo + count, lhi):
+            for i in range(llo + count, lhi):
                 yield _compline(
                     type=type,
                     leftlineno=i + 1,
@@ -735,7 +738,7 @@ def _getcompblockgen(context, leftlines, rightlines, opcodes):
                     rightline=None,
                 )
         elif len2 > len1:
-            for i in pycompat.xrange(rlo + count, rhi):
+            for i in range(rlo + count, rhi):
                 yield _compline(
                     type=type,
                     leftlineno=None,
@@ -860,7 +863,7 @@ class sessionvars(templateutil.wrapped):
 
     def itermaps(self, context):
         separator = self._start
-        for key, value in sorted(pycompat.iteritems(self._vars)):
+        for key, value in sorted(self._vars.items()):
             yield {
                 b'name': key,
                 b'value': pycompat.bytestr(value),
